/* adv_data.js
======================================================================
==   Sage++ (Higmmer's Edition)                                     ==
==     Copyright (C) Higmmer.NEeT / Higmmer All Rights Reserved.    ==
==     The original Sage is the work of The Sage Project.           ==
======================================================================
*/


/************************** Class Hierarchy **************************
 Data --+-- CheckboxData
        +-- TextboxData --- SbmAddUrlTextboxData
        +-- UriSchemeData
        +-- ContainerData --+-- ListData --+-- CssFilterListData
                            |              +-- AdBlockTitleListData
                            |              +-- AdBlockUrlListData
                            |              +-- IgnoreEnclosureListData
                            |              +-- SbmUrlFilterListData
                            +-- TreeData ----- HtmlFilterTreeData
*********************************************************************/


///
///==== Base Classes =================================================
///

///---- Data ---------------------------------------------------------
function Data(id, prefName, prefType, defaultValue) {
	if(arguments.length == 0) return;
	if(arguments.length < 3) throw_FailToCreateData("Missing arguments");
	this.id = id;
	this.target = $(id);
	if(!this.target) throw_FailToCreateData("Invalid target ID");
	this.prefName = prefName;
	this.prefType = prefType;
	this.defaultValue = defaultValue;
}
Data.prototype.init = function(settings) {
	logMessage("init: " + this.id);
}
Data.prototype.accept = function(settings) {
	logMessage("accept: " + this.id);
}
Data.prototype.getPref = function(defaultValue) {
	try{
		var value;
		if(this.defaultValue !== undefined) {
			value = (typeof this.defaultValue == "function") ? this.defaultValue() : this.defaultValue;
		}
		else{
			value = defaultValue;
		}
		return CommonFunc.getPrefValue(this.prefName, this.prefType, value);
	}
	catch(e){
		logMessage("Fail to get pref: " + this.prefName);
		return defaultValue;
	}
}
Data.prototype.setPref = function(value) {
	try{
		CommonFunc.setPrefValue(this.prefName, this.prefType, value);
	}
	catch(e){
		logMessage("Fail to set pref: " + this.prefName);
	}
}


///--- CheckboxData --------------------------------------------------
function CheckboxData(id, prefName, prefType, defaultValue) {
	Data.apply(this, arguments);
}
CheckboxData.prototype = new Data();
CheckboxData.prototype.init = function(settings) {
	this.target.checked = this.getPref(false);
}
CheckboxData.prototype.accept = function(settings) {
	this.setPref(this.target.checked);
}


///--- TextboxData ---------------------------------------------------
function TextboxData(id, prefName, prefType, defaultValue) {
	Data.apply(this, arguments);
}
TextboxData.prototype = new Data();
TextboxData.prototype.init = function(settings) {
	this.target.value = this.getPref("");
}
TextboxData.prototype.accept = function(settings) {
	this.setPref(this.target.value);
}


///--- ContainerData -------------------------------------------------
function ContainerData(id, prefName, prefType, defaultValue) {
	Data.apply(this, arguments);
}
ContainerData.prototype = new Data();
ContainerData.prototype.addItem = function(item, selectedItem) {
	if(!selectedItem) this.getBody().appendChild(item);
	else this.getBody().insertBefore(item, selectedItem);
	if(this.getRowCount() == 1){
		this.disableButtons(false);
		this.select(0);
	}
}
ContainerData.prototype.addNewItem = function(settings) {
	return this.editItem(null, settings);
}
ContainerData.prototype.editItem = function(item, settings) {
	throw_NotImplemented("ContainerData.editItem");
}
ContainerData.prototype.removeItem = function(item) {
	var lastIndex = this.getSelectedIndex();
	this.ensureVisible(lastIndex);
	this.getBody().removeChild(item);
	if(this.getRowCount() == 0) this.disableButtons(true);
	else this.select(Math.min(lastIndex, this.getRowCount() - 1));
}
ContainerData.prototype.ensureVisible = function(index) {
	throw_NotImplemented("ContainerData.ensureVisible");
}
ContainerData.prototype.select = function(index) {
	throw_NotImplemented("ContainerData.select");
}
ContainerData.prototype.getSelectedIndex = function() {
	throw_NotImplemented("ContainerData.getSelectedIndex");
}
ContainerData.prototype.getSelectedItem = function() {
	throw_NotImplemented("ContainerData.getSelectedItem");
}
ContainerData.prototype.getRowCount = function() {
	throw_NotImplemented("ContainerData.getRowCount");
}
ContainerData.prototype.getItemAtIndex = function(index) {
	throw_NotImplemented("ContainerData.getItemAtIndex");
}
ContainerData.prototype.getBody = function() {
	throw_NotImplemented("ContainerData.getBody");
}
ContainerData.prototype.insertItem = function(newItem, oldItem) {
	if(newItem){
		var lastIndex = this.getSelectedIndex();
		if(lastIndex >= 0) this.ensureVisible(lastIndex);
		this.addItem(newItem, this.getSelectedItem());
		if(oldItem) this.removeItem(oldItem);
		if(lastIndex >= 0 && lastIndex != this.getSelectedIndex()) this.select(lastIndex);
		return true;
	}
	return false;
}
ContainerData.prototype.rebuildData = function(settings) {
	throw_NotImplemented("ContainerData.rebuildData");
}
ContainerData.prototype.disableButtons = function(disabled) {
	$("btnEdit_" + this.id).disabled = disabled;
	$("btnRemove_" + this.id).disabled = disabled;
}


///--- ListData ------------------------------------------------------
function ListData(id, prefName, prefType, defaultValue) {
	ContainerData.apply(this, arguments);
	if(arguments.length > 0) this.resetItems();
}
ListData.prototype = new ContainerData();
ListData.prototype.ensureVisible = function(index) {
	this.target.ensureIndexIsVisible(index);
}
ListData.prototype.select = function(index) {
	this.target.selectedIndex = index;
}
ListData.prototype.getSelectedIndex = function() {
	return this.target.selectedIndex;
}
ListData.prototype.getSelectedItem = function() {
	return this.target.selectedItem;
}
ListData.prototype.getRowCount = function() {
	return this.target.getRowCount();
}
ListData.prototype.getItemAtIndex = function(index) {
	return this.target.getItemAtIndex(index);
}
ListData.prototype.getBody = function() {
	return this.target;
}
ListData.prototype.createItem = function(label, value) {
	var listitem = $N("listitem");
	listitem.setAttribute("label", label);
	if(value !== undefined) listitem.setAttribute("value", value);
	return listitem;
}
ListData.prototype.setupItems = function(array) {
	for each(var label in array) this.addItem(this.createItem(label));
	if(this.getRowCount() > 0) this.select(0);
}
ListData.prototype.resetItems = function() {
	while(this.target.hasChildNodes()) this.target.removeChild(this.target.firstChild);
}
ListData.prototype.editItemBase = function(item, labelRes, descRes, validator, acceptor) {
	var _this = this;
	var newItem = null;
	
	openEditDialog(Boolean(item), labelRes, [ item ? item.label : "" ], descRes, validator
		,function(value){
			var result = acceptor(value);
			if(result == "OK") newItem = _this.createItem(value._1);
			return result;
		}
	);
	
	return this.insertItem(newItem, item);
}
ListData.prototype.rebuildDataArray = function(dataArray) {
	dataArray.splice(0);
	for(var i = 0; i < this.getRowCount(); i++){
		dataArray.push(this.getItemAtIndex(i).label);
	}
	dataArray.sort();
}


///--- TreeData ------------------------------------------------------
function TreeData(id, prefName, prefType, defaultValue) {
	ContainerData.apply(this, arguments);
}
TreeData.prototype = new ContainerData();
TreeData.prototype.ensureVisible = function(index) {
	this.target.treeBoxObject.ensureRowIsVisible(index);
}
TreeData.prototype.select = function(index) {
	this.target.view.selection.select(index);
}
TreeData.prototype.getSelectedIndex = function() {
	return this.target.view.selection.currentIndex;
}
TreeData.prototype.getSelectedItem = function() {
	try{ return this.getItemAtIndex(this.getSelectedIndex()); } catch(e){ return null; }
}
TreeData.prototype.getRowCount = function() {
	return this.target.view.rowCount;
}
TreeData.prototype.getItemAtIndex = function(index) {
	return this.target.view.getItemAtIndex(index);
}
TreeData.prototype.getBody = function() {
	return $(this.id + "Body");
}


///
///=== Deliverd Classes ==============================================
///

///--- HTMLFilterTreeData --------------------------------------------
function HtmlFilterTreeData(id, prefName, prefType, defaultValue) {
	TreeData.apply(this, arguments);
}
HtmlFilterTreeData.prototype = new TreeData();
HtmlFilterTreeData.prototype.init = function(settings) {
	loadHTMLElementAndAttributeFilter(settings.tagFilter, settings.attrFilter);
	if(settings.attrFilter[all_tags_id]) settings.tagFilter.push("*");
	settings.tagFilter.sort();
	for each(var tag in settings.tagFilter){
		this.addItem(this.createItem(tag, settings.attrFilter));
	}
}
HtmlFilterTreeData.prototype.accept = function(settings) {
	var result = [];
	for each(var tag in settings.tagFilter){
		var attr = settings.attrFilter[this.getTagId(tag)];
		result.push(tag + (attr ? "{" + attr.join(",") + "}" : ""));
	}
	this.setPref(result.join(" "));
}
HtmlFilterTreeData.prototype.createItem = function(tag, attrFilter) {
	var treeitem = $N("treeitem");
	var treerow = $N("treerow");
	var treecellElem = $N("treecell");
	treecellElem.setAttribute("label", tag);
	var treecellAttr = $N("treecell");
	var attr = attrFilter[this.getTagId(tag)];
	treecellAttr.setAttribute("label", attr ? attr.sort().join(", ") : "");
	
	treerow.appendChild(treecellElem);
	treerow.appendChild(treecellAttr);
	treeitem.appendChild(treerow);
	
	return treeitem;
}
HtmlFilterTreeData.prototype.editItem = function(item, settings) {
	var _this = this;
	var newItem = null;
	
	openEditDialog(Boolean(item)
		,[ "AS_DLG_HTMLFILTER_ELEM_LABEL", "AS_DLG_HTMLFILTER_ATTR_LABEL" ]
		,item ? [ this.getTag(item), this.getAttr(item) ] : [ "", "" ]
		,[ "AS_DLG_HTMLFILTER_DESC1", "AS_DLG_HTMLFILTER_DESC2" ]
		,function(value){
			return /^([a-z0-9]+|\*)$/i.test(value._1) && (!value._2 || /^[a-z0-9-]+(\s*,\s*[a-z0-9-]+)*$/i.test(value._2));
		}
		,function(value){
			value._1 = value._1.toUpperCase();
			value._2 = value._2.toLowerCase().replace(/\s+/g, "");
			if(settings.tagFilter.indexOf(value._1) != -1){
				if(!item || _this.getTag(item) != value._1) return "AS_DLG_ALERT_DUPLICATED";
			}
			var tag = value._1;
			var attrFilter = [];
			if(value._2) attrFilter[_this.getTagId(tag)] = value._2.split(",");
			newItem = _this.createItem(tag, attrFilter);
			return "OK";
		}
	);
	
	return this.insertItem(newItem, item);
}
HtmlFilterTreeData.prototype.rebuildData = function(settings) {
	settings.tagFilter = [];
	settings.attrFilter = [];
	for(var i = 0; i < this.getRowCount(); i++){
		var tag = this.getTag(this.getItemAtIndex(i));
		var attr = this.getAttr(this.getItemAtIndex(i));
		settings.tagFilter.push(tag);
		if(attr){
			settings.attrFilter[this.getTagId(tag)] = attr.replace(/\s+/g, "").split(",");
		}
	}
	settings.tagFilter.sort();
}
HtmlFilterTreeData.prototype.getTag = function(item) {
	return this.target.view.getCellText(this.target.view.getIndexOfItem(item),
	                                    this.target.columns.getColumnAt(0));
}
HtmlFilterTreeData.prototype.getTagId = function(tag) {
	return (tag != "*") ? tag : all_tags_id;
}
HtmlFilterTreeData.prototype.getAttr = function(item) {
	return this.target.view.getCellText(this.target.view.getIndexOfItem(item),
	                                    this.target.columns.getColumnAt(1));
}


///--- UriSchemeData -------------------------------------------------
function UriSchemeData(id, prefName, prefType, defaultValue) {
	Data.apply(this, arguments);
}
UriSchemeData.prototype = new Data();
UriSchemeData.prototype.init = function(settings) {
	if(!settings.schemeFilter) settings.schemeFilter = loadUriSchemeFilter().sort();
	
	if(this.id.match(/chkUriScheme-([a-z]+)/)){
		if(settings.schemeFilter.indexOf(RegExp.$1) != -1) this.target.checked = true;
	}
	else{
		var others = settings.schemeFilter.filter(function(scheme){
			return !/^(http|https|ftp)$/i.test(scheme);
		});
		if(others.length > 0){
			this.target.value = others.join(" ");
			$("chkUriSchemeOthers").checked = true;
		}
		settings.schemeFilter = [];
	}
}
UriSchemeData.prototype.accept = function(settings) {
	if(this.id.match(/chkUriScheme-([a-z]+)/)){
		if(this.target.checked) settings.schemeFilter.push(RegExp.$1);
	}
	else{
		if($("chkUriSchemeOthers").checked){
			settings.schemeFilter = settings.schemeFilter.concat(this.target.value.toLowerCase()
				.replace(/[:/]/g, "").replace(/^[\s,;]+|[\s,;]+$/g, "").split(/[\s,;]+/).sort()
				.filter(function(scheme){
					return !/^(http|https|ftp)$/i.test(scheme);
				})
			);
		}
		this.setPref(settings.schemeFilter.join(" "));
	}
}


///--- CssFilterListData ---------------------------------------------
function CssFilterListData(id, prefName, prefType, defaultValue) {
	ListData.apply(this, arguments);
}
CssFilterListData.prototype = new ListData();
CssFilterListData.prototype.init = function(settings) {
	if(CommonFunc.prefHasUserValue(this.prefName)){
		this.setupItems(settings.cssFilter = splitRegExp(this.getPref(styleRejectionDef)).sort());
	}
	else{
		settings.cssFilter = [];
	}
}
CssFilterListData.prototype.accept = function(settings) {
	this.setPref(settings.cssFilter.join("|"));
}
CssFilterListData.prototype.editItem = function(item, settings) {
	return this.editItemBase(item
		,[ "AS_DLG_CSSFILTER_LABEL" ]
		,[]
		,regexpValidator
		,function(value){
			return regexpAcceptor(value, item, settings.cssFilter);
		}
	);
}
CssFilterListData.prototype.rebuildData = function(settings) {
	this.rebuildDataArray(settings.cssFilter);
}


///--- AdBlockTitleListData ------------------------------------------
function AdBlockTitleListData(id, prefName, prefType, defaultValue) {
	ListData.apply(this, arguments);
}
AdBlockTitleListData.prototype = new ListData();
AdBlockTitleListData.prototype.init = function(settings) {
	this.setupItems(settings.adBlockTitle = (getAdBlockTitleFilter() || []).sort());
}
AdBlockTitleListData.prototype.accept = function(settings) {
	this.setPref(settings.adBlockTitle.join(" "));
}
AdBlockTitleListData.prototype.editItem = function(item, settings) {
	return this.editItemBase(item
		,[ "AS_DLG_ADBLOCK_TITLE_LABEL" ]
		,[ "AS_DLG_ADBLOCK_TITLE_DESC1", "AS_DLG_ADBLOCK_TITLE_DESC2" ]
		,function(value){
			return value._1 && /^\S+$/.test(value._1);
		}
		,function(value){
			value._1 = value._1.toUpperCase();
			return textAcceptor(value, item, settings.adBlockTitle);
		}
	);
}
AdBlockTitleListData.prototype.rebuildData = function(settings) {
	this.rebuildDataArray(settings.adBlockTitle);
}


///--- AdblockUrlListData --------------------------------------------
function AdBlockUrlListData(id, prefName, prefType, defaultValue) {
	ListData.apply(this, arguments);
}
AdBlockUrlListData.prototype = new ListData();
AdBlockUrlListData.prototype.init = function(settings) {
	this.setupItems(settings.adBlockUrl = splitRegExp(getAdBlockUrlFilter()).sort());
}
AdBlockUrlListData.prototype.accept = function(settings) {
	this.setPref(settings.adBlockUrl.join("|"));
}
AdBlockUrlListData.prototype.editItem = function(item, settings) {
	return this.editItemBase(item
		,[ "AS_DLG_ADBLOCK_URL_LABEL" ]
		,[ "AS_DLG_ADBLOCK_URL_DESC1" ]
		,regexpValidator
		,function(value){
			return regexpAcceptor(value, item, settings.adBlockUrl);
		}
	);
}
AdBlockUrlListData.prototype.rebuildData = function(settings) {
	this.rebuildDataArray(settings.adBlockUrl);
}


///--- IgnoreEnclosureListData ---------------------------------------
function IgnoreEnclosureListData(id, prefName, prefType, defaultValue) {
	ListData.apply(this, arguments);
}
IgnoreEnclosureListData.prototype = new ListData();
IgnoreEnclosureListData.prototype.init = function(settings) {
	this.setupItems(settings.ignoreEnclosures = (getIgnoreEnclosures() || []).sort());
}
IgnoreEnclosureListData.prototype.accept = function(settings) {
	this.setPref(settings.ignoreEnclosures.join(" "));
}
IgnoreEnclosureListData.prototype.editItem = function(item, settings) {
	return this.editItemBase(item
		,[ "AS_DLG_IGNORE_ENCLOSURE_LABEL" ]
		,[ "AS_DLG_IGNORE_ENCLOSURE_DESC1", "AS_DLG_IGNORE_ENCLOSURE_DESC2" ]
		,function(value){
			return value._1 && /^!?[^.\s]+$|^\.$/.test(value._1);
		}
		,function(value){
			value._1 = value._1.toLowerCase();
			return textAcceptor(value, item, settings.ignoreEnclosures);
		}
	);
}
IgnoreEnclosureListData.prototype.rebuildData = function(settings) {
	this.rebuildDataArray(settings.ignoreEnclosures);
}


///--- SbmAddUrlTextboxData ------------------------------------------
function SbmAddUrlTextboxData(id, prefName, prefType, defaultValue) {
	TextboxData.apply(this, arguments);
}
SbmAddUrlTextboxData.prototype = new TextboxData();
SbmAddUrlTextboxData.prototype.init = function(settings) {
	var url = this.getPref("");
	this.target.value = /^https?:/.test(url) ? url : this.defaultValue();
}


///--- SbmUrlFilterListData ------------------------------------------
function SbmUrlFilterListData(id, prefName, prefType, defaultValue) {
	ListData.apply(this, arguments);
}
SbmUrlFilterListData.prototype = new ListData();
SbmUrlFilterListData.prototype.init = function(settings) {
	this.setupItems(settings.sbmUrlFilter = splitRegExp(this.getPref(CreateHTML.getSbmUrlFilterDefault())).sort());
}
SbmUrlFilterListData.prototype.accept = function(settings) {
	this.setPref(settings.sbmUrlFilter.join("|"));
}
SbmUrlFilterListData.prototype.editItem = function(item, settings) {
	return this.editItemBase(item
		,[ "AS_DLG_SBM_URL_FILTER_LABEL" ]
		,[ "AS_DLG_SBM_URL_FILTER_DESC1" ]
		,regexpValidator
		,function(value){
			return regexpAcceptor(value, item, settings.sbmUrlFilter);
		}
	);
}
SbmUrlFilterListData.prototype.rebuildData = function(settings) {
	this.rebuildDataArray(settings.sbmUrlFilter);
}


///
///=== Validators & Acceptors ========================================
///

function textAcceptor(value, item, dataArray) {
	if(dataArray.indexOf(value._1) != -1){
		return (item && item.label == value._1) ? "OK" : "AS_DLG_ALERT_DUPLICATED";
	}
	return "OK";
}

function regexpValidator(value) {
	return value._1 && !/^[|\s]+\S*|\S*[|\s]+$/.test(value._1);
}

function regexpAcceptor(value, item, dataArray) {
	if(dataArray.indexOf(value._1) != -1){
		return (item && item.label == value._1) ? "OK" : "AS_DLG_ALERT_DUPLICATED";
	}
	try{
		var re = new RegExp(value._1);
		if(splitRegExp(re).length > 1) return "AS_DLG_ALERT_SPLIT_REGEXP";
	}
	catch(e){
		return "AS_DLG_ALERT_INVALID_REGEXP";
	}
	return "OK";
}


/* End of File */
